eval('/*\n * Copyright (c) 2012 Adobe Systems Incorporated. All rights reserved.\n *  \n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"), \n * to deal in the Software without restriction, including without limitation \n * the rights to use, copy, modify, merge, publish, distribute, sublicense, \n * and/or sell copies of the Software, and to permit persons to whom the \n * Software is furnished to do so, subject to the following conditions:\n *  \n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *  \n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, \n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER \n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING \n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER \n * DEALINGS IN THE SOFTWARE.\n * \n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, regexp: true, indent: 4, maxerr: 50 */\n/*global define, $, brackets */\n\n\n\ndefine(\'main\',[\'require\',\'exports\',\'module\'],function (require, exports, module) {\n    "use strict";\n\n    var EditorManager       = brackets.getModule("editor/EditorManager"),\n        QuickOpen           = brackets.getModule("search/QuickOpen"),\n        JSUtils             = brackets.getModule("language/JSUtils"),\n        DocumentManager     = brackets.getModule("document/DocumentManager"),\n        StringMatch         = brackets.getModule("utils/StringMatch");\n\n\n   /** \n    * FileLocation class\n    * @constructor\n    * @param {string} fullPath\n    * @param {number} line\n    * @param {number} chFrom column start position\n    * @param {number} chTo column end position\n    * @param {string} functionName\n    */\n    function FileLocation(fullPath, line, chFrom, chTo, functionName) {\n        this.fullPath = fullPath;\n        this.line = line;\n        this.chFrom = chFrom;\n        this.chTo = chTo;\n        this.functionName = functionName;\n    }\n\n    /**\n     * Contains a list of information about functions for a single document.\n     *\n     * @return {?Array.<FileLocation>}\n     */\n    function createFunctionList() {\n        var doc = DocumentManager.getCurrentDocument();\n        if (!doc) {\n            return;\n        }\n\n        var functionList = [];\n        var docText = doc.getText();\n        var lines = docText.split("\\n");\n        var functions = JSUtils.findAllMatchingFunctionsInText(docText, "*");\n        functions.forEach(function (funcEntry) {\n            var chFrom = lines[funcEntry.lineStart].indexOf(funcEntry.name);\n            var chTo = chFrom + funcEntry.name.length;\n            functionList.push(new FileLocation(null, funcEntry.lineStart, chFrom, chTo, funcEntry.name));\n        });\n        return functionList;\n    }\n\n    \n\n    /**\n     * @param {string} query what the user is searching for\n     * @param {StringMatch.StringMatcher} matcher object that caches search-in-progress data\n     * @return {Array.<SearchResult>} sorted and filtered results that match the query\n     */\n    function search(query, matcher) {\n        var functionList = matcher.functionList;\n        if (!functionList) {\n            functionList = createFunctionList();\n            matcher.functionList = functionList;\n        }\n        query = query.slice(query.indexOf("@") + 1, query.length);\n        \n        // Filter and rank how good each match is\n        var filteredList = $.map(functionList, function (fileLocation) {\n            var searchResult = matcher.match(fileLocation.functionName, query);\n            if (searchResult) {\n                searchResult.fileLocation = fileLocation;\n            }\n            return searchResult;\n        });\n        \n        // Sort based on ranking & basic alphabetical order\n        StringMatch.basicMatchSort(filteredList);\n\n        return filteredList;\n    }\n\n    /**\n     * @param {string} query what the user is searching for\n     * @param {boolean} returns true if this plug-in wants to provide results for this query\n     */\n    function match(query) {\n        // only match @ at beginning of query for now\n        return (query[0] === "@");\n    }\n\n    /**\n     * Scroll to the selected item in the current document (unless no query string entered yet,\n     * in which case the topmost list item is irrelevant)\n     * @param {?SearchResult} selectedItem\n     * @param {string} query\n     * @param {boolean} explicit False if this is only highlighted due to being at top of list after search()\n     */\n    function itemFocus(selectedItem, query, explicit) {\n        if (!selectedItem || (query.length < 2 && !explicit)) {\n            return;\n        }\n        var fileLocation = selectedItem.fileLocation;\n\n        var from = {line: fileLocation.line, ch: fileLocation.chFrom};\n        var to = {line: fileLocation.line, ch: fileLocation.chTo};\n        EditorManager.getCurrentFullEditor().setSelection(from, to, true);\n    }\n\n    function itemSelect(selectedItem, query) {\n        itemFocus(selectedItem, query, true);\n    }\n\n\n\n    QuickOpen.addQuickOpenPlugin(\n        {\n            name: "JavaScript functions",\n            languageIds: ["javascript"],\n            search: search,\n            match: match,\n            itemFocus: itemFocus,\n            itemSelect: itemSelect\n        }\n    );\n\n});\n//# sourceURL=/main.js');
//# sourceMappingURL=main.js
//# sourceMappingURL=main.js.map